/*
 * Sample "inline" I/O macro for Micro-C/PC
 *
 * These macros insert 80x86 IN/OUT instruction directly inline in your
 * code, thereby avoiding the overhead of calling library functions to
 * perform I/O. This should only be necessary in **VERY** critical real
 * time intensive I/O operations, and will not be required by most users.
 *
 * These macros take advantage of a Micro-C feature that expressions
 * occuring by themselves (with no result used) will evaluate to a value
 * in the accumulator. This is used to obtain the port and data values
 * in C, for use by the inline assembly code.
 *
 * Notes on out/outw:
 *  Since "data" could reference a function call, or contain an expression
 *  containing '*', '/' or '%', any of which could destroy the contents of
 *  the DX register. "port" is saved on the stack during the procssing of
 *  data. If you are certain that your data will not fall into the above
 *  catagory, you can replace the PUSH AX with MOV DX,AX, and eliminate the
 *  POP AX from the out() and outw() macros.
 *
 * Notes on in/inw():
 *  Since a multi-statement macro cannot be embedded in an expression, you
 *  cannot return a value from it. These macros use a "dest" parameter which
 *  specifies a pre-amble to the expression generating the final result. This
 *  allows some flexibility in handling the input value:
 *
 *    in(a=, 0x100);      /* Read port 0x100 and assign value to 'a' */
 *    inw(return, 0x200); /* Return word read from port 0x200 */
 *
 * The function call to the null accumulator access function nargs() will
 * be removed by the MCP optimizer.
 * If you are certain that you will not use byte values read from ports as
 * word values, you can eliminate the XOR AH,AH from the in() macro.
 *
 * Copyright 1997 Dave Dunfield
 * All rights reserved.
 */

#define out(port, data) port; asm {\
 PUSH AX\
} (char)data; asm {\
 POP DX\
 OUT DX,AL\
}

#define outw(port, data) port; asm {\
 PUSH AX\
} (int)data; asm {\
 POP DX\
 OUT DX,AX\
}

#define in(dest, port) port; asm {\
 MOV DX,AX\
 IN AL,DX\
 XOR AH,AH\
} dest nargs();

#define inw(dest, port) port; asm {\
 MOV DX,AX\
 IN AX,DX\
} dest nargs();
